<?php

namespace FiloBlu\ExtElasticsearch\Plugin;

use Magento\Elasticsearch\SearchAdapter\Query\Builder\Match;
use Magento\Framework\Search\Request\QueryInterface as RequestQueryInterface;

/**
 * Class BuilderMatchPlugin
 * @package FiloBlu\ExtElasticsearch\Plugin
 */
class BuilderMatchPlugin
{
    /**
     * @var \Psr\Log\LoggerInterface
     */
    protected $_logger;

    /**
     * BuilderMatchPlugin constructor.
     * @param \Psr\Log\LoggerInterface $logger
     */
    public function __construct(\Psr\Log\LoggerInterface $logger)
    {
        $this->_logger = $logger;
    }

    /**
     * @param Match $subject
     * @param array $selectQuery
     * @param RequestQueryInterface $requestQuery
     * @param $conditionType
     * @return mixed
     */
    public function beforeBuild(Match $subject, array $selectQuery, RequestQueryInterface $requestQuery, $conditionType)
    {
        if (!method_exists($requestQuery, 'getMatches')) {
            return null;
        }

        /** @var $requestQuery \Magento\Framework\Search\Request\Query\Match */
        $matches = $requestQuery->getMatches();

        $filtered = [];

        foreach ($matches as $match) {
            if ($match['field'] === '*') {
                continue;
            }

            $filtered[] = $match;
        }

        return [$selectQuery,
            new \Magento\Framework\Search\Request\Query\Match(
                $requestQuery->getName(),
                $requestQuery->getValue(),
                $requestQuery->getBoost(),
                $filtered), $conditionType];
    }

    /**
     * @param Match $subject
     * @param array $result
     *
     * @return array
     */
    public function afterBuild(Match $subject, $result)
    {
        array_walk_recursive($result, function (&$value, $key) {
            if ($key === 'sku' && is_string($key)) {
                $value = str_replace('\-', '-', $value);
            }
        });

        return $result;
    }
}